# from collections import defaultdict
# import sys 
# import os

# from helper_functions import convert_to_bases
# from bcf_vcf import clean_files, handle_infiles


# def Annotate_with_ref_genome(vcffiles, obsfile):


#     obs = defaultdict(list)

#     with open(obsfile) as data:
#         for line in data:
#             if not line.startswith('chrom'):
#                 chrom, pos, ancestral_base, genotype = line.strip().split()
#                 derived_variant = genotype.replace(ancestral_base, '')[0]
#                 ID = f'{chrom}_{pos}'
#                 obs[ID] = [ancestral_base, derived_variant, '']

#     for vcffile in handle_infiles(vcffiles):
#         command = f'bcftools view -a -R {obsfile} {vcffile}'
#         for line in os.popen(command):
#             if line.startswith('#CHROM'):
#                 individuals_in_vcffile = line.strip().split()[9:]

#             if not line.startswith('#'):

#                 chrom, pos, _, ref_allele, alt_allele = line.strip().split()[0:5]
#                 ID =  f'{chrom}_{pos}'
#                 genotypes = [x.split(':')[0] for x in line.strip().split()[9:]]

#                 ancestral_base, derived_base, _ = obs[ID]
#                 found_in = []

#                 for original_genotype, individual in zip(genotypes, individuals_in_vcffile):

#                     if '.' not in original_genotype:
#                         genotype = convert_to_bases(original_genotype, ref_allele, alt_allele)   

#                         if genotype.count(derived_base) > 0:
#                             found_in.append(individual)
#                             #print(pos, individual, original_genotype)

#                 obs[ID] = [ancestral_base, derived_base, ','.join(found_in)]


#     # Clean log files generated by vcf and bcf tools
#     clean_files('out.log')

#     return obs





# Annotate_with_ref_genome('archaicvar/highcov_ind_1.bcf', 'obs/obs.HG00096.txt')



import argparse

from helper_functions import Maketestdata, make_mutation_rate  
from hmm_functions import TrainModel, DecodeModel
from bcf_vcf import make_out_group, make_ingroup_obs


def print_script_usage():
    toprint = '''
    Script for identifying introgressed archaic segments

    Turorial:
    hmmix make_test_data 
    hmmix train  -obs=obs.txt -weights=weights.bed -mutrates=mutrates.bed -param=Initialguesses.json -out=trained.json 
    hmmix decode -obs=obs.txt -weights=weights.bed -mutrates=mutrates.bed -param=trained.json


    Turorial with 1000 genomes data:
    hmmix create_outgroup -ind=individuals.json -vcf=*.bcf -weights=strickmask.bed -out=outgroup.txt -ancestral=homo_sapiens_ancestor_GRCh37_e71/homo_sapiens_ancestor_*.fa
    hmmix mutation_rate -outgroup=outgroup.txt  -weights=strickmask.bed -window_size=1000000 -out mutationrate.bed
    hmmix create_ingroup  -ind=individuals.json -vcf=*.bcf -weights=strickmask.bed -out=obs -outgroup=outgroup.txt -ancestral=homo_sapiens_ancestor_GRCh37_e71/homo_sapiens_ancestor_*.fa
    
    hmmix train  -obs=obs.HG00096.txt -weights=strickmask.bed -mutrates=mutationrate.bed -out=trained.HG00096.json 
    hmmix decode -obs=obs.HG00096.txt -weights=strickmask.bed -mutrates=mutationrate.bed -param=trained.HG00096.json 

    '''

    return toprint





# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# Main
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
def main():

    parser = argparse.ArgumentParser(description=print_script_usage(), formatter_class=argparse.RawTextHelpFormatter)

    subparser = parser.add_subparsers(dest = 'mode')

    # Run test
    test_subparser = subparser.add_parser('make_test_data', help='Create test data')

    # Make outgroup
    outgroup_subparser = subparser.add_parser('create_outgroup', help='Create outgroup information')
    outgroup_subparser.add_argument("-ind",help="ingroup/outgrop list (json file)", type=str, required = True)
    outgroup_subparser.add_argument("-vcf",help="path to vcf/bcf file(s) - accepts wildcard characters!", type=str, required = True)
    outgroup_subparser.add_argument("-weights", metavar='',help="file with callability (defaults to all positions being called)")
    outgroup_subparser.add_argument("-out", metavar='',help="outputfile (defaults to stdout)", default = '/dev/stdout')
    outgroup_subparser.add_argument("-ancestral", metavar='',help="fasta file with ancestral information (default none)", default='')

    # Make mutation rates
    mutation_rate = subparser.add_parser('mutation_rate', help='Estimate mutation rate')
    mutation_rate.add_argument("-outgroup", help="path to variants found in outgroup", type=str, required = True)
    mutation_rate.add_argument("-out", metavar='',help="outputfile (defaults to mutationrate.bed)", default = 'mutationrate.bed')
    mutation_rate.add_argument("-weights", metavar='',help="file with callability (defaults to all positions being called)")
    mutation_rate.add_argument("-window_size", metavar='',help="size of bins (defaults to 1 Mb)", type=int, default = 1000000)

    # Make ingroup observations
    create_obs_subparser = subparser.add_parser('create_ingroup', help='Create ingroup data')
    create_obs_subparser.add_argument("-ind", help="ingroup/outgrop list (json file)", type=str, required = True)
    create_obs_subparser.add_argument("-vcf", help="path to vcf/bcf file(s)", type=str, required = True)
    create_obs_subparser.add_argument("-outgroup", help="path to variant found in outgroup", type=str, required = True)
    create_obs_subparser.add_argument("-weights", metavar='',help="file with callability (defaults to all positions being called)")
    create_obs_subparser.add_argument("-out", metavar='',help="outputfile prefix (default is a file named obs.<ind>.txt where ind is the name of individual in ingroup/outgrop list)", default = 'obs')
    create_obs_subparser.add_argument("-ancestral", metavar='',help="fasta file with ancestral information (default none)", default='')

    # Train model
    train_subparser = subparser.add_parser('train', help='Train HMM')
    train_subparser.add_argument("-obs",help="file with observation data", type=str, required = True)
    train_subparser.add_argument("-weights", metavar='',help="file with callability (defaults to all positions being called)")
    train_subparser.add_argument("-mutrates", metavar='',help="file with mutation rates (default is mutation rate is uniform)")
    train_subparser.add_argument("-param", metavar='',help="markov parameters file (default is human/neanderthal like parameters)", type=str)
    train_subparser.add_argument("-out", metavar='',help="outputfile prefix (default is a file named trained.json)", default = 'trained.json')
    train_subparser.add_argument("-window_size", metavar='',help="size of bins (default is 1000 bp)", default = 1000)
    train_subparser.add_argument("-haploid",help="Change from using dipliod data to haploid data", action='store_true', default = False)

    # Decode model
    decode_subparser = subparser.add_parser('decode', help='Decode HMM')
    decode_subparser.add_argument("-obs",help="file with observation data", type=str, required = True)
    decode_subparser.add_argument("-weights", metavar='',help="file with callability (defaults to all positions being called)")
    decode_subparser.add_argument("-mutrates", metavar='',help="file with mutation rates (default is mutation rate is uniform)")
    decode_subparser.add_argument("-param", metavar='',help="markov parameters file (default is human/neanderthal like parameters)", type=str)
    decode_subparser.add_argument("-out", metavar='',help="outputfile prefix (default is stdout)", default = '/dev/stdout')
    decode_subparser.add_argument("-window_size", metavar='',help="size of bins (default is 1000 bp)", default = 1000)
    decode_subparser.add_argument("-decode_per_bin",help="get the posterior probability for each window", action='store_true', default = False)
    decode_subparser.add_argument("-haploid",help="Change from using dipliod data to haploid data", action='store_true', default = False)
    decode_subparser.add_argument("-admixpop",help="Annotate using vcffile with admixing population")


    args = parser.parse_args()

    if args.mode == 'make_test_data':
        print('making test data...')
        Maketestdata(data_set_length = 50000)

    elif args.mode == 'train':
        print('training...')
        TrainModel(args.obs, args.weights, args.mutrates, args.param, args.out, args.window_size, args.haploid)

    elif args.mode == 'decode':
        print('decoding...')
        DecodeModel(args.obs, args.weights, args.mutrates, args.param, args.out, args.window_size, args.haploid, args.decode_per_bin, args.admixpop)

    elif args.mode == 'create_outgroup':
        print('Making outgroup...')
        make_out_group(args.ind, args.weights, args.vcf, args.out, args.ancestral)

    elif args.mode == 'create_ingroup':
        print('making ingroup...')
        make_ingroup_obs(args.ind, args.weights, args.vcf, args.out, args.outgroup, args.ancestral)

    elif args.mode == 'mutation_rate':
        print('estimate mutation rate...')
        make_mutation_rate(args.outgroup, args.out, args.weights, args.window_size)


    else:
        print(print_script_usage())


if __name__ == "__main__":
    main()




from collections import defaultdict
import numpy as np
from numba import njit
import math

from helper_functions import Load_observations_weights_mutrates, Load_HMM_parameters, logoutput, Make_HMM_parameters, find_runs, Annotate_with_ref_genome, Make_folder_if_not_exists, Get_genome_coordinates, find_admixed_variants

# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# HMM functions
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------

def Emission_probs_poisson(emissions, observations, weights, mutrates):
    n = len(observations)
    n_states = len(emissions)
    
    # observations values
    fractorials = np.zeros(n)
    for i, obs in enumerate(observations):
        fractorials[i] = math.factorial(obs)

    probabilities = np.zeros( (n, n_states) ) 
    for state in range(n_states): 
        probabilities[:,state] = (np.exp( - emissions[state] * weights * mutrates) *  ((emissions[state] * weights * mutrates )**observations )) / fractorials

    return probabilities

@njit
def fwd_step(alpha_prev, E, trans_mat):
    alpha_new = (alpha_prev @ trans_mat) * E
    n = np.sum(alpha_new)
    return alpha_new / n, n

@njit
def forward(probabilities, transitions, init_start):

    n = len(probabilities)
    forwards_in = np.zeros( (n, len(init_start)) ) 
    scale_param = np.ones(n)

    for t in range(n):
        if t == 0:
            forwards_in[t,:]=  init_start  * probabilities[t,:]
            scale_param[t] = np.sum( forwards_in[t,:])
            forwards_in[t,:] = forwards_in[t,:] / scale_param[t]
        else:
            forwards_in[t,:], scale_param[t] =  fwd_step(forwards_in[t-1,:], probabilities[t,:], transitions) 

    return forwards_in, scale_param
    

@njit
def bwd_step(beta_next, E, trans_mat, n):
    beta = (trans_mat * E) @ beta_next
    return beta / n

@njit
def backward(emissions, transitions, scales):
    n, n_states = emissions.shape
    beta = np.ones((n, n_states))
    for i in range(n - 1, 0, -1):
        beta[i - 1,:] = bwd_step(beta[i,:], emissions[i,:], transitions, scales[i])
    return beta


def GetProbability(starting_probabilities, transitions_matrix, emissions_matrix, weights, obs, mutrates):

    emissions = Emission_probs_poisson(emissions_matrix, obs, weights, mutrates)
    forward_probs, scales = forward(emissions, transitions_matrix, starting_probabilities)
    forward_probility_of_obs = np.sum(np.log(scales))

    return forward_probility_of_obs


def TrainBaumWelsch(starting_probabilities, transitions_matrix, emissions_matrix, weights, obs, mutrates):
    """
    Trains the model once, using the forward-backward algorithm. 
    """

    n_states = len(starting_probabilities)

    emissions = Emission_probs_poisson(emissions_matrix, obs, weights, mutrates)
    forward_probs, scales = forward(emissions, transitions_matrix, starting_probabilities)
    backward_probs = backward(emissions, transitions_matrix, scales)

    # Update emission
    new_emissions_matrix = np.zeros((n_states))
    for state in range(n_states):
        top = np.sum(forward_probs[:, state] * backward_probs[:, state] * obs)
        bottom = np.sum(forward_probs[:, state] * backward_probs[:, state] * (weights * mutrates) )
        new_emissions_matrix[state] = top/bottom


    # Update starting probs
    posterior_probs = forward_probs * backward_probs
    normalize = np.sum(posterior_probs)
    new_starting_probabilities = np.sum(posterior_probs, axis=0)/normalize 


    # Update Transition probs 
    new_transitions_matrix =  np.zeros((n_states, n_states))
    for state1 in range(n_states):
        for state2 in range(n_states):
            new_transitions_matrix[state1,state2] = np.sum( forward_probs[:-1,state1]  * transitions_matrix[state1, state2] * emissions[1:,state2] * backward_probs[1:,state2] / scales[1:] )

    new_transitions_matrix /=  new_transitions_matrix.sum(axis=1)[:,np.newaxis]


    return (new_starting_probabilities, new_transitions_matrix, new_emissions_matrix) 


# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# Train
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
def TrainModel(obs_file, weights_file, mutation_rates_file, parameters_file, outfile, window_size, haploid):

    obs, mutrates, weights = Load_observations_weights_mutrates(obs_file, mutation_rates_file, weights_file, window_size, haploid)
    state_names, transitions, emissions_matrix, starting_probabilities = Load_HMM_parameters(parameters_file)

    print('-' * 40)
    print('Selected parameters')
    print('> State names:', state_names)
    print('> Starting_probabilities:', starting_probabilities)
    print('> Transition matrix:')
    print(transitions)
    print('> Emission values:',emissions_matrix)   
    print()
    print('> number of obs:', len(obs))
    print('> total callability:', round(np.sum(weights) / len(obs),2) )
    print('> average mutation rate per bin:', round(np.sum(mutrates * weights) / np.sum(weights), 2) )
    print('> Output is',outfile) 
    print('-' * 40)

    epsilon = 1e-4

    previous_loglikelihood = GetProbability(starting_probabilities, transitions, emissions_matrix, weights, obs, mutrates)
    logoutput(emissions_matrix, previous_loglikelihood, transitions, starting_probabilities, 0)
    
    for i in range(1,1000):
        starting_probabilities, transitions, emissions_matrix = TrainBaumWelsch(starting_probabilities, transitions, emissions_matrix, weights, obs, mutrates)
        new_loglikelihood = GetProbability(starting_probabilities, transitions, emissions_matrix, weights, obs, mutrates)
        logoutput(emissions_matrix, new_loglikelihood, transitions, starting_probabilities, i)

        if new_loglikelihood - previous_loglikelihood < epsilon:       
            break 

        previous_loglikelihood = new_loglikelihood

    # Write the optimal parameters
    Make_HMM_parameters(state_names, starting_probabilities, transitions, emissions_matrix, outfile)

# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# Decode
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------

def DecodeModel(obs_file, weights_file, mutation_rates_file, parameters_file, output, window_size, haploid, decode_per_bin, admixpop):
    
    obs, mutrates, weights = Load_observations_weights_mutrates(obs_file, mutation_rates_file, weights_file, window_size, haploid)
    state_names, transitions, emissions_matrix, starting_probabilities = Load_HMM_parameters(parameters_file)
    chroms, starts, variants = Get_genome_coordinates(obs_file, window_size, haploid)


    print('-' * 40)
    print('Selected parameters')
    print('> State names:', state_names)
    print('> Starting_probabilities:', starting_probabilities)
    print('> Transition matrix:')
    print(transitions)
    print('> Emission values:',emissions_matrix)   
    print()
    print('> number of obs:', len(obs), '. Number of snps = ', sum(obs))
    print('> total callability:', round(np.sum(weights) / len(obs),2) )
    print('> average mutation rate per bin:', round(np.sum(mutrates * weights) / np.sum(weights), 2) )
    print('> Output is',output) 
    print('-' * 40)


    # Posterior decode the file
    emissions = Emission_probs_poisson(emissions_matrix, obs, weights, mutrates)
    forward_probs, scales = forward(emissions, transitions, starting_probabilities)
    backward_probs = backward(emissions, transitions, scales)

    post_seq = (forward_probs * backward_probs).T

    # write summary file 
    if not decode_per_bin:
        Make_folder_if_not_exists(output)
       
        with open(output,'w') as out: 

            if admixpop is not None:
                admix_pop_variants, admixpop_names = Annotate_with_ref_genome(admixpop, obs_file)
                header = 'chrom\tstart\tend\tlength\tstate\tmean_prob\tsnps\tadmixpopvariants\t{}\n'.format('\t'.join(admixpop_names))
            else:
                header = 'chrom\tstart\tend\tlength\tstate\tmean_prob\tsnps\n'

            out.write(header)

            for (chrom, chrom_start_index, chrom_length_index) in find_runs(chroms):

                state_with_highest_prob = np.argmax(post_seq[:,chrom_start_index:chrom_start_index + chrom_length_index-1], axis = 0)

                for (state, start_index, length_index) in find_runs(state_with_highest_prob):

                    start_index = start_index + chrom_start_index
                    end_index = start_index + length_index

                    genome_start = starts[start_index]
                    genome_end = starts[start_index + length_index - 1]
                    genome_length =  length_index * window_size

                    snp_counter = np.sum(obs[start_index:end_index])
                    mean_prob = np.mean(post_seq[state, start_index:end_index])


                    if admixpop is not None:
                        mylist = find_admixed_variants(variants[start_index:end_index])
                        archiac_variants_dict = defaultdict(int)
                        for snp_position in mylist:
                            variant = admix_pop_variants[f'{chrom}_{snp_position}']
                            if variant != '':
                                if '|' in variant:
                                    for ind in variant.split('|'):
                                        archiac_variants_dict[ind] += 1
                                else:
                                    archiac_variants_dict[variant] += 1

                                archiac_variants_dict['total'] += 1

                        archaic_variants = '\t'.join([str(archiac_variants_dict[x]) for x in ['total'] + admixpop_names])
                        print(chrom, genome_start, genome_end, genome_length, state_names[state], round(mean_prob, 5), snp_counter, archaic_variants, sep = '\t', file = out)
                    else:
                        print(chrom, genome_start, genome_end, genome_length, state_names[state], round(mean_prob, 5), snp_counter, sep = '\t', file = out)



    # write posterios prob for each bin (if specified)    
    if decode_per_bin:

        if admixpop is not None:
            header = 'chrom\tstart\tobservations\tMostlikely\t{}\tvariants\tadmixpopvariants\n'.format('\t'.join(state_names))
            admix_pop_variants, admixpop_names = Annotate_with_ref_genome(admixpop, obs_file)
        else:
            header = 'chrom\tstart\tobservations\tMostlikely\t{}\tvariants\n'.format('\t'.join(state_names))

        Make_folder_if_not_exists(output)
        with open(output,'w') as out:
            out.write(header)
            state_with_highest_prob = np.argmax(post_seq, axis = 0)
            for index, (chrom, start_position, variant_in_bin, obs_in_bin) in enumerate(zip(chroms, starts, variants, obs)):

                state = state_names[state_with_highest_prob[index]]
                probability_of_states = '\t'.join([str(round(val,4)) for val in post_seq[:, index]])

                if admixpop is not None:
                    admix_pop_variants_window = '|'.join([admix_pop_variants[f'{chrom}_{x}'] for x in variant_in_bin.split(',')])
                    print(chrom, index * window_size, obs_in_bin, state, probability_of_states, variant_in_bin, admix_pop_variants_window, sep = '\t', file =  out)
                else:
                    print(chrom, index * window_size, obs_in_bin, state, probability_of_states, variant_in_bin, sep = '\t', file =  out)





import numpy as np
import json
from collections import defaultdict
import os
import itertools
import difflib
from glob import glob

# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# Make test data
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
def Maketestdata(data_set_length):
    '''Create test data set of size data_set_length. Also create uniform weights and uniform mutation rates'''
    np.random.seed(42)


    window_size = 1000
    mutation_rate_window = 1000000

    # Initialization parameters (prob of staring in states)
    state_values = [0,1]
    state_names, transitions, emissions, starting_probabilities = get_default_HMM_parameters()

    print('creating 2 chromosomes with 50 Mb of test data (100K bins) with the following parameters..\n')
    print('State names:', state_names)
    print('Starting_probabilities:', starting_probabilities)
    print('Transition matrix:')
    print(transitions)
    print('Emission values:',emissions)   


    mutation_matrix = {
        'A': [0, 0.16, 0.68, 0.16],
        'C': [0.16, 0,0.16, 0.68],
        'G': [0.68, 0.16, 0, 0.16],
        'T': [0.16, 0.68, 0.16, 0],
    }

    bases = ['A','C','G','T']

    # Make obs file
    with open('obs.txt','w') as obs:

        print('chrom', 'pos', 'ancestral_base', 'genotype', sep = '\t', file = obs)

        for chrom in ['chr1', 'chr2']:
            for index in range(data_set_length):
                if index == 0:
                    current_state = np.random.choice(state_values, p=starting_probabilities)
                else:
                    current_state = np.random.choice(state_values, p=transitions[prevstate] )

                n_mutations = np.random.poisson(lam=emissions[current_state]) 
                for mutation in [int(x) for x in np.random.uniform(low=index*window_size, high=index*window_size + window_size, size=n_mutations)]: 
                    ancestral_base = np.random.choice(bases, p=[0.31, 0.19, 0.19, 0.31])
                    derived_base = np.random.choice(bases, p=mutation_matrix[ancestral_base])
                    print(chrom, mutation, ancestral_base, ancestral_base + derived_base, sep = '\t', file = obs)          

                prevstate = current_state


    # Make mutation file
    with open('mutrates.bed','w') as mutrates:
        for chrom in ['chr1', 'chr2']:
            for start in range(int(data_set_length * window_size / mutation_rate_window)):
                print(chrom, start * mutation_rate_window, (start + 1) * mutation_rate_window, 1, sep = '\t', file = mutrates)



    # Make weights file
    with open('weights.bed','w') as weights:
        for chrom in ['chr1', 'chr2']:
            print(chrom, 1, data_set_length * window_size, sep = '\t', file = weights)



    # Make initial guesses
    state_names = np.array(['Human', 'Archaic'])
    starting_probabilities = np.array([0.5, 0.5])
    transitions = np.array([[0.99,0.01],[0.02,0.98]])
    emissions = np.array([0.03, 0.3])

    Make_HMM_parameters(state_names, starting_probabilities, transitions, emissions, 'Initialguesses.json')

# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# Functions for handling HMM parameters
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
def get_default_HMM_parameters():
    '''Creates human/neanderthal introgression like HMM parameters'''
    state_names = np.array(['Human', 'Archaic'])
    starting_probabilities = np.array([0.98, 0.02])
    transitions = np.array([[0.9999,0.0001],[0.02,0.98]])
    emissions = np.array([0.04, 0.4])

    return state_names, transitions, emissions, starting_probabilities



def Make_HMM_parameters(state_names, starting_probabilities, transitions, emissions, outfile):
    '''Saves parameters to a file'''
    json_string = json.dumps({
                'state_names' : state_names.tolist(),
                'starting_probabilities' : starting_probabilities.tolist(),
                'transitions' : transitions.tolist(),
                'emissions' : emissions.tolist(),
             }, indent = 2)

    Make_folder_if_not_exists(outfile)
    with open(outfile, 'w') as out:
        out.write(json_string)


def Load_HMM_parameters(markov_param):
    '''Loads parameters to a file'''
    if markov_param is None:
        state_names, transitions, emissions, starting_probabilities = get_default_HMM_parameters()
    else:
        with open(markov_param) as json_file:
            data = json.load(json_file)

        state_names, starting_probabilities, transitions, emissions = data['state_names'], data['starting_probabilities'], data['transitions'], data['emissions']


    # convert into numpy arrays
    transitions, starting_probabilities, emissions, state_names = np.array(transitions), np.array(starting_probabilities), np.array(emissions), np.array(state_names)

    return state_names, transitions, emissions, starting_probabilities


# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# Functions for handling observertions/bed files
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
def convert_to_bases(genotype, ref, alt):

    both_bases = ref + alt

    if '|' in genotype:
        base1, base2 = [int(x) for x in genotype.split('|')]
        return both_bases[base1] + both_bases[base2]
    elif '/':
        base1, base2 = [int(x) for x in genotype.split('/')]
        return both_bases[base1] + both_bases[base2]
    



def make_callability_from_bed(bedfile, window_size):
    callability = defaultdict(lambda: defaultdict(float))
    with open(bedfile) as data:
        for line in data:

            if not line.startswith('chrom'):

                if len(line.strip().split('\t')) == 3:
                    chrom, start, end = line.strip().split('\t')
                    value = 1
                elif  len(line.strip().split('\t')) > 3:
                    chrom, start, end, value = line.strip().split('\t')[0:4]
                    value = float(value)

                start = int(start)
                end = int(end)


                firstwindow = start - start % window_size
                firstwindow_fill = window_size - start % window_size

                lastwindow = end - end % window_size
                lastwindow_fill = end %window_size

                # not spanning windows
                if firstwindow == lastwindow:
                    callability[chrom][firstwindow] += (end-start+1) * value

                else:

                    callability[chrom][firstwindow] += firstwindow_fill * value
                    callability[chrom][lastwindow] += (lastwindow_fill+1) * value

                    for window_tofil in range(firstwindow + window_size, lastwindow, window_size):
                        callability[chrom][window_tofil] += window_size * value

    return callability



def Load_observations_weights_mutrates(obs_file, mutrates_file, weights_file, window_size, haploid):

    obs_counter = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))
    if haploid:
        haplotypes = defaultdict(int)
        with open(obs_file) as data:
            for line in data:
                if not line.startswith('chrom'):
                    chrom, pos, ancestral_base, genotype = line.strip().split()
                    rounded_pos = int(pos) - int(pos) % window_size
                    for i, base in enumerate(genotype):
                        if base != ancestral_base:
                            obs_counter[chrom][rounded_pos][f'haploid_{i+1}'] += 1
                            haplotypes[f'haploid_{i+1}'] += 1
    else:
        haplotypes = defaultdict(int)
        with open(obs_file) as data:
            for line in data:
                if not line.startswith('chrom'):
                    chrom, pos, ancestral_base, genotype = line.strip().split()
                    rounded_pos = int(pos) - int(pos) % window_size
                    obs_counter[chrom][rounded_pos]['diploid'] += 1
                    haplotypes['diploid'] += 1

    temp_obs = []
    for haplotype in sorted(haplotypes):
        for chrom in sorted(obs_counter, key=sortby):
            lastwindow = max(obs_counter[chrom]) + window_size

            for window in range(0, lastwindow, window_size):
                temp_obs.append(obs_counter[chrom][window][haplotype])

    # Read weights file is it exists - else set all weights to 1
    if weights_file is None:
        weights = np.ones(len(temp_obs)) 
    else:  
        callability = make_callability_from_bed(weights_file, window_size)
        weights = []
        for haplotype in sorted(haplotypes):
            for chrom in sorted(obs_counter, key=sortby):
                lastwindow = max(obs_counter[chrom]) + window_size

                for window in range(0, lastwindow, window_size):
                    weights.append(callability[chrom][window] / float(window_size))


    # Read mutation rate file is it exists - else set all mutation rates to 1
    if mutrates_file is None:
        mutrates = np.ones(len(temp_obs)) 
    else:  
        callability = make_callability_from_bed(mutrates_file, window_size)
        mutrates = []
        for haplotype in sorted(haplotypes):
            for chrom in sorted(obs_counter, key=sortby):
                lastwindow = max(obs_counter[chrom]) + window_size

                for window in range(0, lastwindow, window_size):
                    mutrates.append(callability[chrom][window] / float(window_size))

    # Make sure there are no places with obs > 0 and 0 in mutation rate or weight
    obs = np.zeros(len(temp_obs))
    for index, (observation, w, m) in enumerate(zip(temp_obs, weights, mutrates)):
        if w*m == 0 and observation != 0:
            obs[index] = 0
            print('warning, you had observations but no called bases/no mutation rate')
            print(index, observation, w, m)
        else:
            obs[index] = int(observation)

    return obs.astype(int), np.array(mutrates).astype(float), np.array(weights).astype(float)



def Get_genome_coordinates(obs_file, window_size, haploid):
    

    obs_counter = defaultdict(lambda: defaultdict(lambda: defaultdict(list)))
    if haploid:
        haplotypes = defaultdict(int)
        with open(obs_file) as data:
            for line in data:
                if not line.startswith('chrom'):
                    chrom, pos, ancestral_base, genotype = line.strip().split()
                    rounded_pos = int(pos) - int(pos) % window_size
                    for i, base in enumerate(genotype):
                        if base != ancestral_base:
                            obs_counter[chrom][rounded_pos][f'_hap{i+1}'].append(pos)
                            haplotypes[f'_hap{i+1}'] += 1
    else:
        haplotypes = defaultdict(int)
        with open(obs_file) as data:
            for line in data:
                if not line.startswith('chrom'):
                    chrom, pos, ancestral_base, genotype = line.strip().split()
                    rounded_pos = int(pos) - int(pos) % window_size
                    obs_counter[chrom][rounded_pos][''].append(pos)
                    haplotypes[''] += 1

    chroms, starts, variants = [], [], []
    for haplotype in sorted(haplotypes):
        for chrom in sorted(obs_counter, key=sortby):
            lastwindow = max(obs_counter[chrom]) + window_size

            for window in range(0, lastwindow, window_size):
                chroms.append(f'{chrom}{haplotype}')   
                starts.append(window)
                variants.append(','.join(obs_counter[chrom][window][haplotype]))

    return chroms, starts, variants


# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# For decoding/training
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
def find_runs(inarray):
        """ run length encoding. Partial credit to R rle function. 
            Multi datatype arrays catered for including non Numpy
            returns: tuple (runlengths, startpositions, values) """
        ia = np.asarray(inarray)                # force numpy
        n = len(ia)
        if n == 0: 
            return (None, None, None)
        else:
            y = ia[1:] != ia[:-1]               # pairwise unequal (string safe)
            i = np.append(np.where(y), n - 1)   # must include last element posi
            z = np.diff(np.append(-1, i))       # run lengths
            p = np.cumsum(np.append(0, z))[:-1] # positions
            #return (ia[i], p, z)

            for (a, b, c) in zip(ia[i], p, z):
                yield (a, b, c)


def logoutput(emissions, loglikelihood, transitions, starting_probabilities, iteration):

    if iteration == 0:    
        print_emissions = '\t'.join(['emis{0}'.format(x + 1) for x in range(len(emissions))])
        print_starting_probabilities = '\t'.join(['start{0}'.format(x + 1) for x in range(len(emissions))])
        print_transitions = '\t'.join(['trans{0}_{0}'.format(x + 1) for x in range(len(emissions))])
        print('iteration', 'loglikelihood', print_starting_probabilities, print_emissions, print_transitions, sep = '\t')

    
    print_emissions = '\t'.join([str(x) for x in np.matrix.round(emissions, 4)])
    print_starting_probabilities = '\t'.join([str(x) for x in np.matrix.round(starting_probabilities, 3)])
    print_transitions = '\t'.join([str(x) for x in np.matrix.round(transitions, 4).diagonal()])
    print(iteration, round(loglikelihood, 4), print_starting_probabilities, print_emissions, print_transitions, sep = '\t')


def get_ancestral_from_fasta(ancestral):
    '''
    Read a fasta file with a single chromosome in and return the sequence as a string

    Ancestra.fa

    >seq1
    AGCATCGATCGACTGACTA

    get_ancestral_from_fasta(Ancestra.fa) returns AGCATCGATCGACTGACTA   
    '''

    ancestral_allele = ''
    with open(ancestral) as data:
        for line in data:
            if line.startswith('>'):
                pass
                #seqname = line.strip().replace('>','')
            else:
                ancestral_allele += line.strip().upper()

    return ancestral_allele


def make_mutation_rate(freqfile, outfile, callablefile, window_size):

    print('-' * 40)
    print('Selected parameters')
    print(f'> Outgroupfile is:\n{freqfile}\n')
    print(f'> Outputfile is:\n{outfile}\n')
    print(f'> Callability file is:\n{callablefile}\n')
    print(f'> Window size is:\n{window_size}\n')
    print('-' * 40)

    snps_counts_window = defaultdict(lambda: defaultdict(int))

    with open(freqfile) as data:
        for line in data:
            if not line.startswith('chrom'):
                chrom, pos = line.strip().split()[0:2]
                # _, ref_count = ref_allele_info.split(':')
                # _, alt_count = alt_allele_info.split(':')
                # pos, ref_count, alt_count = int(pos),  int(ref_count), int(alt_count)
                window = pos - pos%window_size
                snps_counts_window[chrom][window] += 1


    mutations = []
    genome_positions = []
    for chrom in sorted(snps_counts_window, key=sortby):
        lastwindow = max(snps_counts_window[chrom]) + window_size

        for window in range(0, lastwindow, window_size):
            mutations.append(snps_counts_window[chrom][window])
            genome_positions.append([chrom, window, window + window_size])

    mutations = np.array(mutations)

    if callablefile is not None:
        callability = make_callability_from_bed(callablefile, window_size)
        callable_region = []
        for chrom in sorted(snps_counts_window, key=sortby):
            lastwindow = max(snps_counts_window[chrom]) + window_size
            for window in range(0, lastwindow, window_size):
                callable_region.append(callability[chrom][window]/window_size)
    else:
        callable_region = np.ones(len(mutations)) * window_size

    genome_mean = np.sum(mutations) / np.sum(callable_region)

    Make_folder_if_not_exists(outfile)
    with open(outfile,'w') as out:
        print('chrom', 'start', 'end', 'mutationrate', sep = '\t', file = out)
        for genome_pos, mut, call in zip(genome_positions, mutations, callable_region):
            chrom, start, end = genome_pos
            if mut * call == 0:
                ratio = 0
            else:
                ratio = round(mut/call/genome_mean, 2)

            print(chrom, start, end, ratio, sep = '\t', file = out)
        

def get_consensus(infiles):
    '''
    Find consensus prefix, postfix and value that changes in set of files:

    myfiles = ['chr1.vcf', 'chr2.vcf', 'chr3.vcf']
    prefix, postfix, values = get_consensus(myfiles) 
    
    prefix=chr
    postfix=.vcf
    values = {1,2,3} -> is a set
    '''
    infiles = [str(x) for x in infiles]

    consensus_strings = defaultdict(int)
    for a, b in itertools.combinations(infiles,2):
        consensus_a = 'START'
        for i,s in enumerate(difflib.ndiff(a, b)):
            if s[0] != ' ':
                consensus_a += ' '
            else:
                consensus_a += s[-1]
        consensus_a += 'END'


        new_joined = '|'.join(consensus_a.split()).replace('START','').replace('END','')
        consensus_strings[new_joined] += 1

    for value in consensus_strings:

        if len(value.split('|')) == 2:
            prefix, postfix = value.split('|')
            matches = len([x for x in infiles if prefix in x and postfix in x])

            if matches == len(infiles):
                values = [x.replace(prefix, '').replace(postfix,'') for x in infiles]
                return prefix, postfix, set(values)

def sortby(x):
    '''
    This function is used in the sorted() function. It will sort first by numeric values, then strings then other symbols

    Usage:
    mylist = ['1', '12', '2', 3, 'MT', 'Y']
    sortedlist = sorted(mylist, key=sortby)
    returns ['1', '2', 3, '12', 'MT', 'Y']
    '''

    lower_case_letters = 'abcdefghijklmnopqrstuvwxyz'
    if x.isnumeric():
        return int(x)
    elif type(x) == str and len(x) > 0:
        if x[0].lower() in lower_case_letters:
            return 1e6 + lower_case_letters.index(x[0].lower())
        else:
            return 2e6
    else:
        return 3e6


def Make_folder_if_not_exists(path):
    '''
    Check if path exists - otherwise make it;
    If path is /path/to/my/file.txt this will check if /path/to/my/ exists.
    '''
    path = os.path.dirname(path)
    if path != '':
        if not os.path.exists(path):
            os.makedirs(path)



def Annotate_with_ref_genome(vcffiles, obsfile):
    obs = defaultdict(list)
    shared_with = defaultdict(str)

    tempobsfile = obsfile + 'temp'

    with open(obsfile) as data, open(tempobsfile,'w') as out:
        for line in data:
            if not line.startswith('chrom'):
                out.write(line)
                chrom, pos, ancestral_base, genotype = line.strip().split()
                derived_variant = genotype.replace(ancestral_base, '')[0]
                ID = f'{chrom}_{pos}'
                obs[ID] = [ancestral_base, derived_variant]

    print('Loading in admixpop snp information')
    for vcffile in handle_infiles(vcffiles):
        command = f'bcftools view -a -R {tempobsfile} {vcffile}'
        print(command)

        for line in os.popen(command):
            if line.startswith('#CHROM'):
                individuals_in_vcffile = line.strip().split()[9:]

            if not line.startswith('#'):

                chrom, pos, _, ref_allele, alt_allele = line.strip().split()[0:5]
                ID =  f'{chrom}_{pos}'
                genotypes = [x.split(':')[0] for x in line.strip().split()[9:]]

                ancestral_base, derived_base = obs[ID]
                found_in = []

                for original_genotype, individual in zip(genotypes, individuals_in_vcffile):

                    if '.' not in original_genotype:
                        genotype = convert_to_bases(original_genotype, ref_allele, alt_allele)   

                        if genotype.count(derived_base) > 0:
                            found_in.append(individual)

                if len(found_in) > 0:
                    shared_with[ID] = '|'.join(found_in)


    # Clean log files generated by vcf and bcf tools
    clean_files('out.log')
    clean_files(tempobsfile)

    return shared_with, individuals_in_vcffile

def get_ingroup_or_outgroup(outgroup_ingroupfile, key):
    '''
    reads a json file which has the keys outgroup and ingroup. In both the values should be a string. 
    '''
    with open(outgroup_ingroupfile) as json_file:
        data = json.load(json_file)

    return data[key]

def handle_individuals_input(argument, group_to_choose):
    # Is this a json file?
    if os.path.exists(argument):
        return get_ingroup_or_outgroup(argument, group_to_choose)
    else:
        return argument.split(',')


# Check which type of input we are dealing with
def handle_infiles(input):
    file_list = glob(input)
    if len(file_list) > 0:
        return file_list
    else:
        if ',' in input:
            return input.split(',')
        else:
            return [input]

# Clean up
def clean_files(filename):
    if os.path.exists(filename):
        os.remove(filename)


# Find variants from admixed population
def find_admixed_variants(variants_list):

    flattened_list = []
    for bin in variants_list:
        if bin != '':
            if ',' in bin:
                for position in bin.split(','):
                    flattened_list.append(position)

            else:
                flattened_list.append(bin)

    return flattened_list





import os
import numpy as np
from collections import defaultdict
import sys 
from glob import glob
import json

from helper_functions import get_consensus, sortby, Make_folder_if_not_exists, get_ancestral_from_fasta, convert_to_bases, handle_individuals_input, handle_infiles, clean_files

# Check which type of input we are dealing with
def combined_files(ancestralfiles, vcffiles):

    if len(ancestralfiles) == len(vcffiles):

        if ancestralfiles == ['']:
            return [None], vcffiles
        else:
            return ancestralfiles, vcffiles



    elif ancestralfiles == ['']:
        if len(vcffiles) > 1:
            prefix1, postfix1, values1 = get_consensus(vcffiles)
            vcffiles = []
            for joined in sorted(values1, key=sortby):
                vcffiles.append(''.join([prefix1, joined, postfix1]))

        ancestralfiles = [None for _ in vcffiles]
        return ancestralfiles, vcffiles
       
    elif len(ancestralfiles) > 1 and len(vcffiles) > 1:
        prefix1, postfix1, values1 = get_consensus(vcffiles)
        prefix2, postfix2, values2 = get_consensus(ancestralfiles)

        vcffiles = []
        ancestralfiles = []

        for joined in sorted(values1.intersection(values2), key=sortby):
            vcffiles.append(''.join([prefix1, joined, postfix1]))
            ancestralfiles.append(''.join([prefix2, joined, postfix2]))
        return ancestralfiles, vcffiles
    
    elif len(ancestralfiles) > 1 and len(vcffiles) == 1:
        prefix2, postfix2, values2 = get_consensus(ancestralfiles)
        ancestralfiles = []
        
        for key in values2:
            if key in vcffiles[0]:
                ancestralfiles.append(''.join([prefix2, key, postfix2]))

        return ancestralfiles, vcffiles

    elif len(ancestralfiles) == 1 and len(vcffiles) > 1:
        prefix1, postfix1, values1 = get_consensus(vcffiles)
        vcffiles = []
        
        for key in values1:
            if key in ancestralfiles[0]:
                vcffiles.append(''.join([prefix1, key, postfix1]))

        if len(vcffiles) > len(ancestralfiles):
            sys.exit('Could not resolve ancestral files and vcffiles (try comma separated values)')


        return ancestralfiles, vcffiles





# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
# Dealing with bcf/vcf files functions
# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
def make_out_group(individuals_input, bedfile, vcffiles, outputfile, ancestralfiles):

    # Get list of outgroup individuals
    outgroup_individuals = handle_individuals_input(individuals_input, 'outgroup')

    # Get a list of vcffiles and ancestral files and intersect them
    vcffiles = handle_infiles(vcffiles)
    ancestralfiles = handle_infiles(ancestralfiles)
    ancestralfiles, vcffiles  = combined_files(ancestralfiles, vcffiles)

    print('-' * 40)
    print('Selected parameters')
    print('\n> Outgroup individuals:', len(outgroup_individuals))
    print('First 10..')
    for i, x in enumerate(outgroup_individuals):
        print(x)
        if i > 10:
            break
    print()
    print('> Using vcf and ancestral files')
    for vcffile, ancestralfile in zip(vcffiles, ancestralfiles):
        print('vcffile:',vcffile, 'ancestralfile:',ancestralfile)
    print()    
    print('> Callability file:\n', bedfile, '\n')
    print(f'> Writing output to:\n',outputfile,'\n')
    print('-' * 40)

    Make_folder_if_not_exists(outputfile)
    outgroup_individuals = ','.join(outgroup_individuals)

    with open(outputfile, 'w') as out:

        print('chrom', 'pos', 'ref_allele_info', 'alt_allele_info', 'ancestral_base', sep = '\t', file = out)

        for vcffile, ancestralfile in zip(vcffiles, ancestralfiles):

            if ancestralfile is not None:
                ancestral_allele = get_ancestral_from_fasta(ancestralfile)

            if bedfile is not None:
                command = f'bcftools view -m2 -M2 -v snps -s {outgroup_individuals} -T {bedfile} {vcffile} | vcftools --vcf - --counts --stdout'
            else:
                command = f'bcftools view -m2 -M2 -v snps -s {outgroup_individuals} {vcffile} | vcftools --vcf - --counts --stdout'
            
            print(f'Processing {vcffile}...')
            print('Running command:')
            print(command, '\n\n')

            for index, line in enumerate(os.popen(command)):

                if not line.startswith('CHROM'):

                    chrom, pos, _, _, ref_allele_info, alt_allele_info = line.strip().split()

                    ref_allele, ref_count = ref_allele_info.split(':')
                    alt_allele, alt_count = alt_allele_info.split(':')

                    pos, ref_count, alt_count = int(pos),  int(ref_count), int(alt_count)

                    # Always include polymorphic sites
                    if alt_count * ref_count > 0:
                        ancestral_base = ref_allele if ref_count > alt_count else alt_allele

                        # Use ancestral base info if available
                        if ancestralfile is not None:
                            ancestral_base_temp = ancestral_allele[pos-1]
                            if ancestral_base_temp in [ref_allele, alt_allele]:
                                 ancestral_base = ancestral_base_temp

                        print(chrom, pos, ref_allele_info, alt_allele_info, ancestral_base, sep = '\t', file = out)

                    # Fixed sites
                    elif alt_count * ref_count == 0:

                        ancestral_base = ref_allele if ref_count > alt_count else alt_allele
                        # Use ancestral base info if available
                        if ancestralfile is not None:
                            ancestral_base_temp = ancestral_allele[pos-1]
                            if ancestral_base_temp in [ref_allele, alt_allele]:
                                 ancestral_base = ancestral_base_temp

                        if ancestral_base == alt_allele:
                            derived_count = ref_count
                        else:
                             derived_count = alt_count

                        if derived_count > 0:
                            print(chrom, pos, ref_allele_info, alt_allele_info, ancestral_base, sep = '\t', file = out)


                    if index % 100000 == 0:
                        print(f'at line {index} at chrom {chrom} and position {pos}')

    # Clean log files generated by vcf and bcf tools
    clean_files('out.log')
    






def make_ingroup_obs(individuals_input, bedfile, vcffiles, outprefix, outgroupfile, ancestralfiles):

    # Get a list of ingroup individuals
    ingroup_individuals = handle_individuals_input(individuals_input, 'ingroup')

    # Get a list of vcffiles and ancestral files and intersect them
    vcffiles = handle_infiles(vcffiles)
    ancestralfiles = handle_infiles(ancestralfiles)
    ancestralfiles, vcffiles  = combined_files(ancestralfiles, vcffiles)

    print('-' * 40)
    print('Selected parameters')
    print('\n > Ingroup individuals:', len(ingroup_individuals))
    print('First 10')
    for i, x in enumerate(ingroup_individuals):
        print(x)
        if i > 10:
            break
    print()

    print('> Using vcf and ancestral files')
    for vcffile, ancestralfile in zip(vcffiles, ancestralfiles):
        print('vcffile:',vcffile, 'ancestralfile:',ancestralfile)
    print()  
    print('> Using outgroup variants from:\n', outgroupfile, '\n')  
    print('> Callability file:\n', bedfile, '\n')
    print(f'Writing output to file with prefix:\n{outprefix}.<individual>.txt\n')
    print('-' * 40)

    
    
    list_of_obs = defaultdict(list)
    ingroup_individuals = ','.join(ingroup_individuals)

    for vcffile, ancestralfile in zip(vcffiles, ancestralfiles):

        if ancestralfile is not None:
            ancestral_allele = get_ancestral_from_fasta(ancestralfile)

        if bedfile is not None:
            command = f'bcftools view -m2 -M2 -v snps -s {ingroup_individuals} -T {bedfile} {vcffile} | vcftools --vcf - --exclude-positions {outgroupfile} --recode --stdout'
        else:
            command = f'bcftools view -m2 -M2 -v snps -s {ingroup_individuals} {vcffile} | vcftools --vcf - --exclude-positions {outgroupfile} --recode --stdout'

        print('Running command:')
        print(command, '\n\n')

        for index, line in enumerate(os.popen(command)):

            if line.startswith('#CHROM'):
                individuals_in_vcffile = line.strip().split()[9:]

            if not line.startswith('#'):

                chrom, pos, _, ref_allele, alt_allele = line.strip().split()[0:5]
                pos = int(pos)
                genotypes = [x.split(':')[0] for x in line.strip().split()[9:]]

                for original_genotype, individual in zip(genotypes, individuals_in_vcffile):
                    ref_count = original_genotype.count('0')
                    alt_count = original_genotype.count('1')     
                    genotype = convert_to_bases(original_genotype, ref_allele, alt_allele)   

                    if ancestralfile is not None:
                        # With ancestral information look for derived alleles
                        ancestral_base = ancestral_allele[pos-1]
                        if ancestral_base in [ref_allele, alt_allele]:

                            derived_count = genotype.count(alt_allele) if ancestral_base == ref_allele else genotype.count(ref_allele)
                            if derived_count > 0:
                                list_of_obs[individual].append([chrom, pos, ancestral_base, genotype])

                    else:
                        # If no ancestral information is provided only include heterozygous variants
                        if alt_count * ref_count > 0:
                            list_of_obs[individual].append([chrom, pos, ancestral_base, genotype])
            

                if index % 100000 == 0:
                    print(f'at line {index} at chrom {chrom} and position {pos}')


    # Write output files
    print('\n\nSummary:')
    Make_folder_if_not_exists(outprefix)
    for individual in ingroup_individuals.split(','):
        summary_counter = defaultdict(int)
        with open(f'{outprefix}.{individual}.txt','w') as out:
            print('chrom', 'pos', 'ancestral_base', 'genotype', sep = '\t', file = out)

            for values in list_of_obs[individual]:
                chrom, pos, ancestral_base, genotype = values
                print(chrom, pos, ancestral_base, genotype, sep = '\t', file = out)

                if len(set(genotype)) == 1:
                    summary_counter['homozygous'] += 1
                else:
                    summary_counter['heterozygous'] += 1

        het = summary_counter['heterozygous']
        hom = summary_counter['homozygous']
        total = het + hom
        print(f'{individual} had {total} variants. {hom} were homozygous and {het} were heterozygous')


    # Clean log files generated by vcf and bcf tools
    clean_files('out.log')



